# frozen_string_literal: true

require 'savon'
require 'countries'

module IntegrationPoint
  class Client

    attr_reader :xml_client, :xml_response

    def xml_client(wsdl_url)
      return nil if wsdl_url.nil?

      @xml_client ||= ::Savon.client(wsdl: wsdl_url, namespace_identifier: 's', follow_redirects: true, convert_request_keys_to: :none)
    end

    def dps_integrated_search(wsdl_url, xml_values)
      client = xml_client(wsdl_url)

      return nil if client.nil?

      query = build_dps_integration_search_query(xml_values)
      @xml_response = client.call(:dps_integrated_search, message: query)
      self
    end

    private

    def build_dps_integration_search_query(xml_values)
      {
        'company': empty_string_if_nil(xml_values[:company]),
        'username': empty_string_if_nil(xml_values[:username]),
        'password': empty_string_if_nil(xml_values[:password]),
        'loggedInUser':  empty_string_if_nil(xml_values[:logged_in_user]),
        'companyId': empty_string_if_nil(xml_values[:company_id]),
        'searchType': empty_string_if_nil(xml_values[:search_type]),
        'settingsDescription': empty_string_if_nil(xml_values[:settings_description]),
        'name': empty_string_if_nil(xml_values[:name]),
        'address': empty_string_if_nil(xml_values[:address]),
        'city': empty_string_if_nil(xml_values[:city]),
        'countryCode': determine_country_code(xml_values[:country]),
        'countryStateCode': determine_state_code(xml_values[:state]),
        'postalCode': empty_string_if_nil(xml_values[:postal_code]),
        'dtsSearchFlag': empty_string_if_nil(xml_values[:dts_search_flag]),
        'dtsLastValidatedDate': empty_string_if_nil(xml_values[:dts_last_validated_date]),
        'dtsOverride': empty_string_if_nil(xml_values[:dts_override]),
        'dtsOverrideDate': empty_string_if_nil(xml_values[:dts_override_date]),
        'SearchRefNum': empty_string_if_nil(xml_values[:search_ref_num]),
        'CompanySync': empty_string_if_nil(xml_values[:company_sync])
      }
    end

    def determine_country_code(country)
      return "" if country.nil?

      @country = ISO3166::Country.find_country_by_name(country)
      if @country.nil?
        ""
      else
        @country.alpha2
      end
    end

    def empty_string_if_nil(value)
      return "" if value.nil?

      value
    end

    def determine_state_code(state)
      return "" if @country.nil?

      states = @country.states
      state_struct = states.detect { |s| s.last.name.downcase == "british columbia".downcase }
      if state_struct.nil?
        ""
      else
        state_struct.first
      end
    end
  end
end
